/*
 *  Copyright 2010 University of Nantes, IRCCyN lab.
 *
 *  Florent Autrusseau.
 *  start : 08/10/2010.
 *  end   : 14/10/2010.
 *
 *  Questions and comments to:
 *       <mailto:florent.autrusseau@univ-nantes.fr>
 *       <http://www.irccyn.ec-nantes.fr/~autrusse/>
 */

/*
 * qcif= 144 x 176
 * cif=  288 x 352
*/

#include "SDL.h"
//#include "SDL_image.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <signal.h>
#include <string.h>
#include <assert.h>

#ifdef _WIN32
#include <windows.h>
#define  Win        1
#else
#include <unistd.h>
#define  Win        0
#endif

#define FALSE	0
#define TRUE	1

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
Uint32 rmask = 0xff000000;
Uint32 gmask = 0x00ff0000;
Uint32 bmask = 0x0000ff00;
Uint32 amask = 0x000000ff;
#else
Uint32 rmask = 0x000000ff;
Uint32 gmask = 0x0000ff00;
Uint32 bmask = 0x00ff0000;
Uint32 amask = 0xff000000;
#endif

#ifdef _WIN32
void msleep(int ms) {
	Sleep(ms);
}
#else
/*----------------------------------------------------*/
/*                    function                        */
/*----------------------------------------------------*/
void msleep(int ms) {
	struct timespec time;
	time.tv_sec = ms / 1000;
	time.tv_nsec = (ms % 1000) * (1000 * 1000);
	nanosleep(&time,NULL);
}
#endif

// MY INCLUDES:
#include "io_stuff.h"

#define SaveImges	0

/**************************************************************************************************/
int main(int argc, char **argv)
{
	int i,j,k;
	int ms_frame_lap = 40;
	int width, height, size;
	int halfwidth, halfheight, halfsize;
	int zoom = 0, inc = 0, inc2 = 0;
	long lSize;
	long int offset;
	long NbFrames;
	char VideoName[256];
	char ImagesName[256];
	char ch_H[100];
	char ch_W[100];
	unsigned char *VidY;
	double *VidU2;
	double *VidV2;
	unsigned char *VidY3;
	double *VidU3;
	double *VidV3;
	unsigned char *VidU;
	unsigned char *VidV;
	size_t result, result2;

	/** SDL variables: **/
	SDL_Surface *screen, *imageO;
    SDL_Event event;

	FILE *Input=NULL;

	// get the arguments, if no arguments, play foreman_qcif.yuv
	if (argc == 1)	{
		sprintf(VideoName,"foreman_qcif.yuv");
		height=144;
		width=176;
	}
	else	{
		sprintf(VideoName,"%s",argv[1]);
		sprintf(ch_H,"%s",argv[2]);
		height=atoi(ch_H);
		sprintf(ch_W,"%s",argv[3]);
		width=atoi(ch_W);
	}

	// get the frames size
	size = height * width;
	halfwidth = (int) (width / 2);
	halfheight = (int) (height / 2);
	halfsize = halfheight * halfwidth;

	/**************** STARTING SDL STUFF ****************/
    /** startup SDL **/
    if(SDL_Init(SDL_INIT_VIDEO)==-1)	{
	printf("SDL_Init: %s\n", SDL_GetError());
	return 1;
    }

    screen=SDL_SetVideoMode(width, height, 24, SDL_ANYFORMAT); 
    if(!screen)	{
	printf("SDL_SetVideoMode: %s\n", SDL_GetError());
	SDL_Quit();
	return 1;
    }

	SDL_FillRect( screen, NULL, SDL_MapRGBA(screen->format, 255, 255, 255, 255) );

    imageO = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, width, height, 24, 0,0,0,0);
	SDL_EnableKeyRepeat(1,1);
	/**************** END OF SDL STUFF ****************/

	Input = fopen(VideoName,"rb");
	if (Input==NULL) {fputs ("File error",stderr); exit (1);}

	// get the file size
	fseek (Input , 0 , SEEK_END);
	lSize = ftell (Input);
	rewind (Input);
	NbFrames = (lSize / (long) (size)) / 1.5;

	printf("%ld frames\n",NbFrames);

	// allocate memory for Y,U,V and U2, V2 (upsampled U&V)
	VidY = (unsigned char*) calloc (size,sizeof(unsigned char));
	if (VidY == NULL) {fputs ("Memory error\n",stderr); exit (2);}
	VidU2 = (double*) calloc (size,sizeof(double));
	if (VidU2 == NULL) {fputs ("Memory error\n",stderr); exit (2);}
	VidV2 = (double*) calloc (size,sizeof(double));
	if (VidV2 == NULL) {fputs ("Memory error\n",stderr); exit (2);}
	VidU = (unsigned char*) calloc (halfsize,sizeof(unsigned char));
	if (VidU == NULL) {fputs ("Memory error\n",stderr); exit (2);}
	VidV = (unsigned char*) calloc (halfsize,sizeof(unsigned char));
	if (VidV == NULL) {fputs ("Memory error\n",stderr); exit (2);}

	VidY3 = (unsigned char*) calloc (4*size,sizeof(unsigned char));
	if (VidY == NULL) {fputs ("Memory error\n",stderr); exit (2);}
	VidU3 = (double*) calloc (4*size,sizeof(double));
	if (VidU3 == NULL) {fputs ("Memory error\n",stderr); exit (2);}
	VidV3 = (double*) calloc (4*size,sizeof(double));
	if (VidV3 == NULL) {fputs ("Memory error\n",stderr); exit (2);}

	int done=0;

	// get events from keyboard, and play/pause/forward/rewind/... video
	while(!done && SDL_WaitEvent(&event)!=-1)
	{
	    switch(event.type)
	    {
		case SDL_KEYDOWN:
		{
		    int handled=0;
		    switch(event.key.keysym.sym)
		    {
		    	// quit
				case SDLK_ESCAPE:
					done = 1;
				    break;
		    	// play video
				case SDLK_RETURN:
				    handled=1;
				    break;
				default:
				    break;
		    }
		    if(handled)
		    {

			// processing every frame 
			k=0;
			while ((k < NbFrames) && (k >= 0))
			{
				offset = k*size;

				fseek(Input, 1.5*offset, 0);
				result = fread (VidY, 1, size, Input);
				if (result != size) {fputs ("Reading error\n",stderr); exit (3);}

				result2 = fread (VidU, 1, halfsize, Input);
				if (result2 != halfsize) {fputs ("Reading error\n",stderr); exit (3);}

				result2 = fread (VidV, 1, halfsize, Input);
				if (result2 != halfsize) {fputs ("Reading error\n",stderr); exit (3);}

				// Resize U and V (x2 upsampling, no interpolation yet)
				for (i=0;i<height;i++)	{
					for (j=0;j<width;j++)	{
						VidU2[i*width+j] = (double) (VidU[(int)(round(i/2))*halfwidth+(int)(round(j/2))] - 128.0);
						VidV2[i*width+j] = (double) (VidV[(int)(round(i/2))*halfwidth+(int)(round(j/2))] - 128.0);
					}
				}

				// if zoomed in once, then, rescale every frame
				if ((zoom == 1) && (inc == 1))	{
					for (i=0;i<2*height;i++)	{
						for (j=0;j<2*width;j++)	{
							VidY3[i*2*width+j] = (double) (VidY[(int)(round(i/2))*width+(int)(round(j/2))]);
							VidU3[i*2*width+j] = (double) (VidU2[(int)(round(i/2))*width+(int)(round(j/2))]);
							VidV3[i*2*width+j] = (double) (VidV2[(int)(round(i/2))*width+(int)(round(j/2))]);
						}
					}
				}

				// Save Y,U,V frames as PGM images
				if (SaveImges)	{
					sprintf(ImagesName,"Images/Yframe_%d.pgm",k);
					WritePGM(VidY, (int)(height), (int)(width), ImagesName);
					sprintf(ImagesName,"Images/Uframe_%d.pgm",k);
					WritePGM(VidU, (int)(height/2), (int)(width/2), ImagesName);
					sprintf(ImagesName,"Images/Vframe_%d.pgm",k);
					WritePGM(VidV, (int)(height/2), (int)(width/2), ImagesName);
					sprintf(ImagesName,"Images/U2frame_%d.tim",k);
					StoreDoubleFile((int)(height/2), (int)(width/2), ImagesName, VidU2);
					sprintf(ImagesName,"Images/V2frame_%d.tim",k);
					StoreDoubleFile((int)(height/2), (int)(width/2), ImagesName, VidV2);
				}

				// Convert YUV2RGB and diaplay the RGB frames using SDL
				if (zoom == 0)	{
					if (inc2 == 0)	{
					    screen=SDL_SetVideoMode(width, height, 24, SDL_ANYFORMAT); 
					    if(!screen)	{
							printf("SDL_SetVideoMode: %s\n", SDL_GetError());
							SDL_Quit();
							return 1;
				  		}
						SDL_FillRect( screen, NULL, SDL_MapRGBA(screen->format, 255, 255, 255, 255) );
					    imageO = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, width, height, 24, 0,0,0,0);
					    inc2++;
					    inc=0;
					}
					// Display the X1 video size
					DisplayOnScreen(imageO, screen, width, height, VidY, VidU2, VidV2, ms_frame_lap);
				}
				else if (zoom == 1)	{
					if (inc == 0)	{
					    screen=SDL_SetVideoMode(2*width, 2*height, 24, SDL_ANYFORMAT); 
						if(!screen)	{
							printf("SDL_SetVideoMode: %s\n", SDL_GetError());
							SDL_Quit();
							return 1;
				    	}
						SDL_FillRect( screen, NULL, SDL_MapRGBA(screen->format, 255, 255, 255, 255) );
				    	imageO = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, 2*width, 2*height, 24, 0,0,0,0);
				    	inc++;
				    	inc2=0;
					}
					// Display the X2 video size
					DisplayOnScreen(imageO, screen, 2*width, 2*height, VidY3, VidU3, VidV3, ms_frame_lap);
				}				

				// Wait for any action on the keyboard...
				SDL_PollEvent(&event);

				// Exit the video
				if (event.key.keysym.sym == SDLK_ESCAPE)	{
					fclose(Input);
					SDL_FreeSurface(screen);
					SDL_FreeSurface(imageO);
					exit(0);
				}
				// Zoom in : make the video height and with 2 times bigger
				else if (event.key.keysym.sym == SDLK_2)	{
					zoom=1;
				}
				// Zoom out : get back to normal frame size
				else if (event.key.keysym.sym == SDLK_1)	{
					zoom=0;
				}
				// speed up the video
				else if (event.key.keysym.sym == SDLK_UP)	{
                     if (Win == 1)
                      	ms_frame_lap = 20;	// bug with MS windows ? 
                      						// for some strange reason, 
                      						// it does not work when using "ms_frame_lap -= 10;" ??
                     else
                      	ms_frame_lap -= 10;   	
					k++;
				}
				// go in slow motion
				else if (event.key.keysym.sym == SDLK_DOWN)	{
					ms_frame_lap += 10;
					k++;
				}
				// forward playing
				else if (event.key.keysym.sym == SDLK_RIGHT)	{
					k++;
				}
				// rewind playing
				else if (event.key.keysym.sym == SDLK_LEFT)	{
					k--;
				}
				// back to the original 40ms between frames speed
				else if (event.key.keysym.sym == SDLK_o)	{
					ms_frame_lap = 40;
					k++;
				}
				// pause the video
				else if (event.key.keysym.sym == SDLK_SPACE)	{
					SDL_WaitEvent(&event);
					if (event.key.keysym.sym == SDLK_s)	{
						sprintf(ImagesName,"Images/Yframe_%d.pgm",k);
						WritePGM(VidY, (int)(height), (int)(width), ImagesName);
						sprintf(ImagesName,"Images/Uframe_%d.pgm",k);
						WritePGM(VidU, (int)(height/2), (int)(width/2), ImagesName);
						sprintf(ImagesName,"Images/Vframe_%d.pgm",k);
						WritePGM(VidV, (int)(height/2), (int)(width/2), ImagesName);
						SDL_WaitEvent(&event);
					}
					else	{
						SDL_WaitEvent(&event);
						k++;
					}
				}
				else	{
					k++;
				}

				// if positionned at the 1st or last frame, waiting for a keyboard event
				if (k == 0)	{
					SDL_WaitEvent(&event);
					k++;
				}
				if (k == NbFrames)	{
					SDL_WaitEvent(&event);
					k--;
				}
			}

			fclose(Input);
			SDL_FreeSurface(screen);
			SDL_FreeSurface(imageO);
			continue;

			// We're done, ESC was pressed, free SDL memory
			if (done == 1)
			{
				SDL_FreeSurface(screen);
				SDL_FreeSurface(imageO);
				//SDL_Quit();
				continue;
			}

			break;
		    }
		}
	    }
	}


	SDL_Quit();
	free(VidY);
	free(VidU);
	free(VidV);
	free(VidU2);
	free(VidV2);
	free(VidV3);
	free(VidU3);
	free(VidV3);

    return 0;
}
